/******************************************************************************
*   SETEST: Demonstrates how to enumerate a list of Windows NT services and   *
*           displays the results in a listbox. The names displayed are the    *
*           names used to control the services, but the structure defined in  *
*           the ListAllServices() function contains a great deal more infor-  *
*           -mation such as the service state and "pretty" name.              *
*******************************************************************************
*                               1998 by Mark Gamber                           *
* By using this program or any part of it, the user assumes full respon-      *
* -sibility for it's use and may not hold the author liable for any loss or   *
* damage. If you cannot accept this restriction, the program may not be used  *
* and must be destroyed immediately.                                          *
******************************************************************************/

#include "windows.h"
#include "windowsx.h"
#include "resource.h"

//  === Function Prototypes ===================================================

BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL ListAllServices( HWND );

// === Global Variables =======================================================

HINSTANCE hInst;

// === Program Entry Point ====================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
					  int nShow )
{
	hInst = hInstance;                               //  Create main dialog box
	DialogBox( hInst, MAKEINTRESOURCE( 10000 ), NULL, MainDlgProc );
	return( FALSE );
}

// === Main Dialog Procedure ==================================================

BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:    //  When the dialog box starts, get all services
			ListAllServices( GetDlgItem( hDlg, 100 ) );    //  store in listbox
			return( TRUE );

		case WM_COMMAND:
			if( wParam == IDCANCEL )           //  Close when cancel is pressed
			{
				EndDialog( hDlg, TRUE );
				return( TRUE );
			}
			break;
	}
	return( FALSE );
}

// === ListAllServices() ======================================================
//  ListAllServices() adds the service name of each found service on the NT box
//     to the listbox identified by the passed handle.

BOOL ListAllServices( HWND hWnd )
{
	SC_HANDLE schSCManager;
	ENUM_SERVICE_STATUS ess;
	ENUM_SERVICE_STATUS *pess, *npess;
	char szName[ 512 ];
	DWORD dwNeeded, dwReturned, dwResume, dwLoop;

                           //  Open the service manager with enumeration rights
	schSCManager = OpenSCManager( NULL, NULL, SC_MANAGER_ENUMERATE_SERVICE );
	if( schSCManager == NULL )                             //  Exit if it fails
		return( FALSE );
                                                       //  Initialize variables
	dwResume = dwNeeded = dwReturned = 0;
	                     //  Call with no buffer to see how much buffer we need
	if( ! EnumServicesStatus( schSCManager, SERVICE_WIN32, SERVICE_STATE_ALL,
							  &ess, sizeof(ENUM_SERVICE_STATUS),
						      &dwNeeded, &dwReturned, &dwResume ) )
		if( GetLastError() != ERROR_MORE_DATA )        //  If failed because of
		{                    //  something other than needing more memory, exit
			CloseServiceHandle( schSCManager );
			return( FALSE );
		}

	dwReturned = 0;
	dwNeeded += 32;            //  Add a few to the needed bytes...just in case
                                                  //  Allocate that much memory
	pess = (ENUM_SERVICE_STATUS *)GlobalAllocPtr( GHND, dwNeeded );
	if( ! pess )
	{                                        // Of course, die on alloc failure
		CloseServiceHandle( schSCManager );
		return( FALSE );
	}
	memset( (LPVOID)pess, 0, dwNeeded - 1 );        //  Initialize memory block
                                          //  Now get the service info for real
	if( EnumServicesStatus( schSCManager, SERVICE_WIN32, SERVICE_STATE_ALL,
							pess, dwNeeded,
							&dwNeeded, &dwReturned, &dwResume ) )
	{
		npess = pess;                       //  Set new pointer to memory block
		for( dwLoop = 0; dwLoop < dwReturned; dwLoop++ )
		{          //  and for each struct, add the service name to the listbox
			SendMessage( hWnd, LB_ADDSTRING, 0, (LPARAM)npess->lpServiceName );
			++npess;                                 //  Move to next structure
		}
	}
	else                                //  If something failed, show the error
	{
		wsprintf( szName, "Error %d in EnumServicesStatus()", GetLastError() );
		MessageBox( GetParent( hWnd ), szName, "Test", MB_OK );
	}

	GlobalFreePtr( pess );                                  //  Free the buffer
	CloseServiceHandle( schSCManager );           //  Close the service manager
	return( TRUE );
}

// ============================================================================


